@@ -73,6 +73,64 @@ int imap_expand_path (char* path, size_t len)
 }
 
 #ifdef USE_HCACHE
+
+/* Generates a seqseq of the UIDs in msn_index to persist in the header cache.
+ *
+ * Empty spots are stored as 0.
+ */
+static void imap_msn_index_to_uid_seqset (BUFFER *b, IMAP_DATA *idata)
+{
+  int first = 1, state = 0, match = 0;
+  HEADER *cur_header;
+  unsigned int msn, cur_uid = 0, last_uid = 0;
+  unsigned int range_begin = 0, range_end = 0;
+
+  for (msn = 1; msn <= idata->max_msn + 1; msn++)
+  {
+    match = 0;
+    if (msn <= idata->max_msn)
+    {
+      cur_header = idata->msn_index[msn - 1];
+      cur_uid = cur_header ? HEADER_DATA(cur_header)->uid : 0;
+      if (!state || (cur_uid && (cur_uid - 1 == last_uid)))
+        match = 1;
+      last_uid = cur_uid;
+    }
+
+    if (match)
+    {
+      switch (state)
+      {
+        case 1:            /* single: convert to a range */
+          state = 2;
+          /* fall through */
+        case 2:            /* extend range ending */
+          range_end = cur_uid;
+          break;
+        default:
+          state = 1;
+          range_begin = cur_uid;
+          break;
+      }
+    }
+    else if (state)
+    {
+      if (first)
+        first = 0;
+      else
+        mutt_buffer_addch (b, ',');
+
+      if (state == 1)
+        mutt_buffer_add_printf (b, "%u", range_begin);
+      else if (state == 2)
+        mutt_buffer_add_printf (b, "%u:%u", range_begin, range_end);
+
+      state = 1;
+      range_begin = cur_uid;
+    }
+  }
+}
+
 static int imap_hcache_namer (const char* path, char* dest, size_t dlen)
 {
   return snprintf (dest, dlen, "%s.hcache", path);
@@ -84,6 +142,7 @@ header_cache_t* imap_hcache_open (IMAP_DATA* idata, const char* path)
   ciss_url_t url;
   char cachepath[LONG_STRING];
   char mbox[LONG_STRING];
+  size_t len;
 
   if (path)
     imap_cachepath (idata, path, mbox, sizeof (mbox));
@@ -96,6 +155,12 @@ header_cache_t* imap_hcache_open (IMAP_DATA* idata, const char* path)
     FREE (&mx.mbox);
   }
 
+  if (strstr(mbox, "/../") || (strcmp(mbox, "..") == 0) || (strncmp(mbox, "../", 3) == 0))
+    return NULL;
+  len = strlen(mbox);
+  if ((len > 3) && (strcmp(mbox + len - 3, "/..") == 0))
+    return NULL;
+
   mutt_account_tourl (&idata->conn->account, &url);
   url.path = mbox;
   url_ciss_tostring (&url, cachepath, sizeof (cachepath), U_PATH);
@@ -158,6 +223,56 @@ int imap_hcache_del (IMAP_DATA* idata, unsigned int uid)
   sprintf (key, "/%u", uid);
   return mutt_hcache_delete (idata->hcache, key, imap_hcache_keylen);
 }
+
+int imap_hcache_store_uid_seqset (IMAP_DATA *idata)
+{
+  BUFFER *b;
+  size_t seqset_size;
+  int rc;
+
+  if (!idata->hcache)
+    return -1;
+
+  b = mutt_buffer_new ();
+  /* The seqset is likely large.  Preallocate to reduce reallocs */
+  mutt_buffer_increase_size (b, HUGE_STRING);
+  imap_msn_index_to_uid_seqset (b, idata);
+
+  seqset_size = b->dptr - b->data;
+  if (seqset_size == 0)
+    b->data[0] = '\0';
+
+  rc = mutt_hcache_store_raw (idata->hcache, "/UIDSEQSET",
+                               b->data, seqset_size + 1,
+                               imap_hcache_keylen);
+  dprint (5, (debugfile, "Stored /UIDSEQSET %s\n", b->data));
+  mutt_buffer_free (&b);
+  return rc;
+}
+
+int imap_hcache_clear_uid_seqset (IMAP_DATA *idata)
+{
+  if (!idata->hcache)
+    return -1;
+
+  return mutt_hcache_delete (idata->hcache, "/UIDSEQSET", imap_hcache_keylen);
+}
+
+char *imap_hcache_get_uid_seqset (IMAP_DATA *idata)
+{
+  char *hc_seqset, *seqset;
+
+  if (!idata->hcache)
+    return NULL;
+
+  hc_seqset = mutt_hcache_fetch_raw (idata->hcache, "/UIDSEQSET",
+                                     imap_hcache_keylen);
+  seqset = safe_strdup (hc_seqset);
+  mutt_hcache_free ((void **)&hc_seqset);
+  dprint (5, (debugfile, "Retrieved /UIDSEQSET %s\n", NONULL (seqset)));
+
+  return seqset;
+}
 #endif
 
 /* imap_parse_path: given an IMAP mailbox name, return host, port
@@ -364,24 +479,14 @@ void imap_error (const char *where, const char *msg)
   mutt_sleep (2);
 }
 
-/* imap_new_idata: Allocate and initialise a new IMAP_DATA structure.
- *   Returns NULL on failure (no mem) */
+/* imap_new_idata: Allocate and initialise a new IMAP_DATA structure. */
 IMAP_DATA* imap_new_idata (void)
 {
   IMAP_DATA* idata = safe_calloc (1, sizeof (IMAP_DATA));
 
-  if (!idata)
-    return NULL;
-
-  if (!(idata->cmdbuf = mutt_buffer_new ()))
-    FREE (&idata);
-
+  idata->cmdbuf = mutt_buffer_new ();
   idata->cmdslots = ImapPipelineDepth + 2;
-  if (!(idata->cmds = safe_calloc(idata->cmdslots, sizeof(*idata->cmds))))
-  {
-    mutt_buffer_free(&idata->cmdbuf);
-    FREE (&idata);
-  }
+  idata->cmds = safe_calloc (idata->cmdslots, sizeof(*idata->cmds));
 
   return idata;
 }
@@ -608,28 +713,35 @@ void imap_qualify_path (char *dest, size_t len, IMAP_MBOX *mx, char* path)
 }
 
 
-/* imap_quote_string: quote string according to IMAP rules:
- *   surround string with quotes, escape " and \ with \ */
-void imap_quote_string (char *dest, size_t dlen, const char *src)
+static void _imap_quote_string (char *dest, size_t dlen, const char *src,
+                                const char *to_quote)
 {
-  static const char quote[] = "\"\\";
   char *pt;
   const char *s;
 
+  if (!(dest && dlen && src && to_quote))
+    return;
+
+  if (dlen < 3)
+  {
+    *dest = 0;
+    return;
+  }
+
   pt = dest;
   s  = src;
 
-  *pt++ = '"';
-  /* save room for trailing quote-char */
-  dlen -= 2;
+  /* save room for pre/post quote-char and trailing null */
+  dlen -= 3;
 
+  *pt++ = '"';
   for (; *s && dlen; s++)
   {
-    if (strchr (quote, *s))
+    if (strchr (to_quote, *s))
     {
+      if (dlen < 2)
+        break;
       dlen -= 2;
-      if (!dlen)
-	break;
       *pt++ = '\\';
       *pt++ = *s;
     }
@@ -643,6 +755,23 @@ void imap_quote_string (char *dest, size_t dlen, const char *src)
   *pt = 0;
 }
 
+/* imap_quote_string: quote string according to IMAP rules:
+ *   surround string with quotes, escape " and \ with \ */
+void imap_quote_string (char *dest, size_t dlen, const char *src)
+{
+  _imap_quote_string (dest, dlen, src, "\"\\");
+}
+
+/* imap_quote_string_and_backquotes: quote string according to IMAP rules:
+ *   surround string with quotes, escape " and \ with \.
+ * Additionally, escape backquotes with \ to protect against code injection
+ * when using the resulting string in mutt_parse_rc_line().
+ */
+void imap_quote_string_and_backquotes (char *dest, size_t dlen, const char *src)
+{
+  _imap_quote_string (dest, dlen, src, "\"\\`");
+}
+
 /* imap_unquote_string: equally stupid unquoting routine */
 void imap_unquote_string (char *s)
 {
@@ -850,3 +979,92 @@ int imap_account_match (const ACCOUNT* a1, const ACCOUNT* a2)
 
   return mutt_account_match (a1_canon, a2_canon);
 }
+
+/* Sequence set iteration */
+
+SEQSET_ITERATOR *mutt_seqset_iterator_new (const char *seqset)
+{
+  SEQSET_ITERATOR *iter;
+
+  if (!seqset || !*seqset)
+    return NULL;
+
+  iter = safe_calloc (1, sizeof(SEQSET_ITERATOR));
+  iter->full_seqset = safe_strdup (seqset);
+  iter->eostr = strchr (iter->full_seqset, '\0');
+  iter->substr_cur = iter->substr_end = iter->full_seqset;
+
+  return iter;
+}
+
+/* Returns: 0 when the next sequence is generated
+ *          1 when the iterator is finished
+ *         -1 on error
+ */
+int mutt_seqset_iterator_next (SEQSET_ITERATOR *iter, unsigned int *next)
+{
+  char *range_sep;
+
+  if (!iter || !next)
+    return -1;
+
+  if (iter->in_range)
+  {
+    if ((iter->down && iter->range_cur == (iter->range_end - 1)) ||
+        (!iter->down && iter->range_cur == (iter->range_end + 1)))
+      iter->in_range = 0;
+  }
+
+  if (!iter->in_range)
+  {
+    iter->substr_cur = iter->substr_end;
+    if (iter->substr_cur == iter->eostr)
+      return 1;
+
+    while (!*(iter->substr_cur))
+      iter->substr_cur++;
+    iter->substr_end = strchr (iter->substr_cur, ',');
+    if (!iter->substr_end)
+      iter->substr_end = iter->eostr;
+    else
+      *(iter->substr_end) = '\0';
+
+    range_sep = strchr (iter->substr_cur, ':');
+    if (range_sep)
+      *range_sep++ = '\0';
+
+    if (mutt_atoui (iter->substr_cur, &iter->range_cur))
+      return -1;
+    if (range_sep)
+    {
+      if (mutt_atoui (range_sep, &iter->range_end))
+        return -1;
+    }
+    else
+      iter->range_end = iter->range_cur;
+
+    iter->down = (iter->range_end < iter->range_cur);
+    iter->in_range = 1;
+  }
+
+  *next = iter->range_cur;
+  if (iter->down)
+    iter->range_cur--;
+  else
+    iter->range_cur++;
+
+  return 0;
+}
+
+void mutt_seqset_iterator_free (SEQSET_ITERATOR **p_iter)
+{
+  SEQSET_ITERATOR *iter;
+
+  if (!p_iter || !*p_iter)
+    return;
+
+  iter = *p_iter;
+  FREE (&iter->full_seqset);
+  FREE (p_iter);               /* __FREE_CHECKED__ */
+}
+
