@@ -140,7 +140,7 @@ int smime_valid_passphrase (void)
   
   if (mutt_get_password (_("Enter S/MIME passphrase:"), SmimePass, sizeof (SmimePass)) == 0)
     {
-      SmimeExptime = time (NULL) + SmimeTimeout;
+      SmimeExptime = mutt_add_timeout (time (NULL), SmimeTimeout);
       return (1);
     }
   else
@@ -1527,11 +1527,13 @@ BODY *smime_sign_message (BODY *a )
   int err = 0;
   int empty = 0;
   pid_t thepid;
-  smime_key_t *default_key;
+  char *signas;
+  smime_key_t *signas_key;
   char *intermediates;
   char *micalg;
 
-  if (!SmimeDefaultKey)
+  signas = (SmimeSignAs && *SmimeSignAs) ? SmimeSignAs : SmimeDefaultKey;
+  if (!signas || !*signas)
   {
     mutt_error _("Can't sign: No key specified. Use Sign As.");
     return NULL;
@@ -1563,22 +1565,22 @@ BODY *smime_sign_message (BODY *a )
   
 
   snprintf (SmimeKeyToUse, sizeof (SmimeKeyToUse), "%s/%s", 
-	   NONULL(SmimeKeys), SmimeDefaultKey);
+	   NONULL(SmimeKeys), signas);
 
   snprintf (SmimeCertToUse, sizeof (SmimeCertToUse), "%s/%s",
-	   NONULL(SmimeCertificates), SmimeDefaultKey);
+	   NONULL(SmimeCertificates), signas);
   
-  default_key = smime_get_key_by_hash (SmimeDefaultKey, 1);
-  if ((! default_key) ||
-      (! mutt_strcmp ("?", default_key->issuer)))
-    intermediates = SmimeDefaultKey; /* so openssl won't complain in any case */
+  signas_key = smime_get_key_by_hash (signas, 1);
+  if ((! signas_key) ||
+      (! mutt_strcmp ("?", signas_key->issuer)))
+    intermediates = signas; /* so openssl won't complain in any case */
   else
-    intermediates = default_key->issuer;
+    intermediates = signas_key->issuer;
 
   snprintf (SmimeIntermediateToUse, sizeof (SmimeIntermediateToUse), "%s/%s",
 	   NONULL(SmimeCertificates), intermediates);
 
-  smime_free_key (&default_key);
+  smime_free_key (&signas_key);
   
 
 
@@ -1839,23 +1841,23 @@ static BODY *smime_handle_entity (BODY *m, STATE *s, FILE *outFile)
     mutt_perror (outfile);
     return NULL;
   }
-  
+
   mutt_mktemp (errfile, sizeof (errfile));
   if ((smimeerr = safe_fopen (errfile, "w+")) == NULL)
   {
     mutt_perror (errfile);
-    safe_fclose (&smimeout); smimeout = NULL;
+    safe_fclose (&smimeout);
     return NULL;
   }
   mutt_unlink (errfile);
 
-  
+
   mutt_mktemp (tmpfname, sizeof (tmpfname));
   if ((tmpfp = safe_fopen (tmpfname, "w+")) == NULL)
   {
     mutt_perror (tmpfname);
-    safe_fclose (&smimeout); smimeout = NULL;
-    safe_fclose (&smimeerr); smimeerr = NULL;
+    safe_fclose (&smimeout);
+    safe_fclose (&smimeerr);
     return NULL;
   }
 
@@ -1870,7 +1872,7 @@ static BODY *smime_handle_entity (BODY *m, STATE *s, FILE *outFile)
       (thepid = smime_invoke_decrypt (&smimein, NULL, NULL, -1,
 				      fileno (smimeout),  fileno (smimeerr), tmpfname)) == -1)
   {
-    safe_fclose (&smimeout); smimeout = NULL;
+    safe_fclose (&smimeout);
     mutt_unlink (tmpfname);
     if (s->flags & MUTT_DISPLAY)
       state_attach_puts (_("[-- Error: unable to create OpenSSL subprocess! --]\n"), s);
@@ -1881,14 +1883,14 @@ static BODY *smime_handle_entity (BODY *m, STATE *s, FILE *outFile)
 					  fileno (smimeout), fileno (smimeerr), NULL,
 					  tmpfname, SIGNOPAQUE)) == -1)
   {
-    safe_fclose (&smimeout); smimeout = NULL;
+    safe_fclose (&smimeout);
     mutt_unlink (tmpfname);
     if (s->flags & MUTT_DISPLAY)
       state_attach_puts (_("[-- Error: unable to create OpenSSL subprocess! --]\n"), s);
     return NULL;
   }
 
-  
+
   if (type & ENCRYPT)
   {
     if (!smime_valid_passphrase ())
@@ -1898,25 +1900,25 @@ static BODY *smime_handle_entity (BODY *m, STATE *s, FILE *outFile)
   }
 
   safe_fclose (&smimein);
-	
+
   mutt_wait_filter (thepid);
   mutt_unlink (tmpfname);
-  
+
 
   if (s->flags & MUTT_DISPLAY)
   {
     fflush (smimeerr);
     rewind (smimeerr);
-    
+
     if ((c = fgetc (smimeerr)) != EOF)
     {
       ungetc (c, smimeerr);
-      
+
       crypt_current_time (s, "OpenSSL");
       mutt_copy_stream (smimeerr, s->fpout);
       state_attach_puts (_("[-- End of OpenSSL output --]\n\n"), s);
     }
-    
+
     if (type & ENCRYPT)
       state_attach_puts (_("[-- The following data is S/MIME"
                            " encrypted --]\n"), s);
@@ -1924,64 +1926,71 @@ static BODY *smime_handle_entity (BODY *m, STATE *s, FILE *outFile)
       state_attach_puts (_("[-- The following data is S/MIME signed --]\n"), s);
   }
 
-  if (smimeout)
+  fflush (smimeout);
+  rewind (smimeout);
+
+  if (type & ENCRYPT)
   {
-    fflush (smimeout);
+    /* void the passphrase, even if that wasn't the problem */
+    if (fgetc (smimeout) == EOF)
+    {
+      mutt_error _("Decryption failed");
+      smime_void_passphrase ();
+    }
     rewind (smimeout);
-    
-    if (outFile) fpout = outFile;
-    else
-    {
-      mutt_mktemp (tmptmpfname, sizeof (tmptmpfname));
-      if ((fpout = safe_fopen (tmptmpfname, "w+")) == NULL)
-      {
-	mutt_perror(tmptmpfname);
-	safe_fclose (&smimeout); smimeout = NULL;
-	return NULL;
-      }
-    }
-    while (fgets (buf, sizeof (buf) - 1, smimeout) != NULL)
-    {
-      len = mutt_strlen (buf);
-      if (len > 1 && buf[len - 2] == '\r')
-      {
-	buf[len-2] = '\n';
-	buf[len-1] = '\0';
-      }
-      fputs (buf, fpout);
-    }
-    fflush (fpout);
-    rewind (fpout); 
-
-
-    if ((p = mutt_read_mime_header (fpout, 0)) != NULL)
-    {
-      fstat (fileno (fpout), &info);
-      p->length = info.st_size - p->offset;
-	  
-      mutt_parse_part (fpout, p);
-      if (s->fpout)
-      {
-	rewind (fpout);
-	tmpfp_buffer = s->fpin;
-	s->fpin = fpout;
-	mutt_body_handler (p, s);
-	s->fpin = tmpfp_buffer;
-      }
-      
-    }
-    safe_fclose (&smimeout);
-    smimeout = NULL;
-    mutt_unlink (outfile);
-
-    if (!outFile)
-    {
-      safe_fclose (&fpout);
-      mutt_unlink (tmptmpfname);
-    }
-    fpout = NULL;
   }
 
+  if (outFile) fpout = outFile;
+  else
+  {
+    mutt_mktemp (tmptmpfname, sizeof (tmptmpfname));
+    if ((fpout = safe_fopen (tmptmpfname, "w+")) == NULL)
+    {
+      mutt_perror(tmptmpfname);
+      safe_fclose (&smimeout);
+      return NULL;
+    }
+  }
+  while (fgets (buf, sizeof (buf) - 1, smimeout) != NULL)
+  {
+    len = mutt_strlen (buf);
+    if (len > 1 && buf[len - 2] == '\r')
+    {
+      buf[len-2] = '\n';
+      buf[len-1] = '\0';
+    }
+    fputs (buf, fpout);
+  }
+  fflush (fpout);
+  rewind (fpout);
+
+  if ((p = mutt_read_mime_header (fpout, 0)) != NULL)
+  {
+    fstat (fileno (fpout), &info);
+    p->length = info.st_size - p->offset;
+
+    mutt_parse_part (fpout, p);
+    if (s->fpout)
+    {
+      rewind (fpout);
+      tmpfp_buffer = s->fpin;
+      s->fpin = fpout;
+      mutt_body_handler (p, s);
+      s->fpin = tmpfp_buffer;
+    }
+
+  }
+
+  safe_fclose (&smimeout);
+  mutt_unlink (outfile);
+
+  if (!outFile)
+  {
+    safe_fclose (&fpout);
+    mutt_unlink (tmptmpfname);
+  }
+  fpout = NULL;
+
   if (s->flags & MUTT_DISPLAY)
   {
     if (type & ENCRYPT)
@@ -1995,15 +2004,15 @@ static BODY *smime_handle_entity (BODY *m, STATE *s, FILE *outFile)
     char *line = NULL;
     int lineno = 0;
     size_t linelen;
-    
+
     rewind (smimeerr);
-    
+
     line = mutt_read_line (line, &linelen, smimeerr, &lineno, 0);
     if (linelen && !ascii_strcasecmp (line, "verification successful"))
       m->goodsig = 1;
     FREE (&line);
   }
-  else 
+  else if (p)
   {
     m->goodsig = p->goodsig;
     m->badsig  = p->badsig;
@@ -2088,7 +2097,16 @@ bail:
 
 int smime_application_smime_handler (BODY *m, STATE *s)
 {
-  return smime_handle_entity (m, s, NULL) ? 0 : -1;
+  int rv = -1;
+  BODY *tattach;
+
+  tattach = smime_handle_entity (m, s, NULL);
+  if (tattach)
+  {
+    rv = 0;
+    mutt_free_body (&tattach);
+  }
+  return rv;
 }
 
 int smime_send_menu (HEADER *msg)
@@ -2216,19 +2234,11 @@ int smime_send_menu (HEADER *msg)
       break;
 
     case 's': /* (s)ign */
+      msg->security &= ~ENCRYPT;
+      msg->security |= SIGN;
+      break;
+
     case 'S': /* (s)ign in oppenc mode */
-      if(!SmimeDefaultKey)
-      {
-        if ((key = smime_ask_for_key (_("Sign as: "), KEYFLAG_CANSIGN, 0)))
-        {
-          mutt_str_replace (&SmimeDefaultKey, key->hash);
-          smime_free_key (&key);
-        }
-        else
-          break;
-      }
-      if (choices[choice - 1] == 's')
-        msg->security &= ~ENCRYPT;
       msg->security |= SIGN;
       break;
 
@@ -2236,9 +2246,9 @@ int smime_send_menu (HEADER *msg)
 
       if ((key = smime_ask_for_key (_("Sign as: "), KEYFLAG_CANSIGN, 0))) 
       {
-        mutt_str_replace (&SmimeDefaultKey, key->hash);
+        mutt_str_replace (&SmimeSignAs, key->hash);
         smime_free_key (&key);
-          
+
         msg->security |= SIGN;
 
         /* probably need a different passphrase */
